searchData={"items":[{"type":"module","title":"Kafkaesque","doc":"This module provides the main APIs for Kafkaesque\n\n#","ref":"Kafkaesque.html"},{"type":"module","title":"Introduction - Kafkaesque","doc":"Kafkaesque is an outbox library built for PostgreSQL and designed, primarily,\nfor Kafka, though usage with other software is possible and encouraged.\n\n- **Transactional safety for messages:** if they were created, they **will** be\neventually published. They're only created if the transaction commits.\n- **Ordering:** messages are published sequentially for topic/partition combinations\n- **Shutdown safety:** has graceful shutdown and rescue for cases where it doesn't\nhappen.\n- **Observability:** all operations publish telemetry events.\n- **Garbage collection:** outbox table is periodically cleaned.\n- **Multi-node safe:** safety powered by PostgreSQL.\n\nFor a comprehensive installation guide, check the [Getting started](getting-started.html)\nguide.\n\n![Basic diagram](http://www.plantuml.com/plantuml/proxy?cache=no&src=https://raw.githubusercontent.com/v0idpwn/kafkaesque/master/diagrams/basic.iuml)\n\n##","ref":"Kafkaesque.html#module-introduction"},{"type":"module","title":"Note - Kafkaesque","doc":"For most cases, this module shouldn't be called directly. Instead, you want to\n`use` it to define an outbox for your application, and call the outbox, as you\ndo with Ecto repos.","ref":"Kafkaesque.html#module-note"},{"type":"function","title":"Kafkaesque.child_spec/1","doc":"Child spec for a Kafkaesque instance","ref":"Kafkaesque.html#child_spec/1"},{"type":"function","title":"Kafkaesque.publish/5","doc":"Publishes a message in the outbox.\n\nWhile you can use this function, this is not the recommended approach to\nuse the library. See the documentation of the `Kafkaesque` module for more\ninformation.","ref":"Kafkaesque.html#publish/5"},{"type":"function","title":"Kafkaesque.start_link/1","doc":"Starts a Kafkaesque instance.\n\nAccepts the following opts:\n\n- `:repo`: the repo where messages will be read from. Usually should be the\nsame repo that you're writing to.\n- `:client`: the client to be used by the publisher. Defaults to\n`Kafkaesque.Clients.BrodClient`\n- `:client_opts`: the options to be used by the client. Defaults to `[]`. The\ndefault client requires options, so this can be considered required for most\nuse-cases. Look at the client documentation for more information about the\nclient options.\n- `:producer_max_backoff_ms`: maximum time in milliseconds that the producer will\ntake between database reads. Notice that this is edge-case scenario and should\nonly happen when a) there are database issues or b) there are no new messages\nin the table for long.\n- `:publisher_max_demand`: maximum publisher demand, can be useful for tuning.\nDefaults to 200. See `GenStage` documentation for more info.\n- `:publisher_min_demand`: minimum publisher demand, can be useful for tuning.\nDefaults to 190. See `GenStage` documentation for more info.\n- `:rescuer_interval_ms`: the interval between garbage collection\nruns. Defaults to 15 seconds. Notice that it always runs on tree startup.\n- `rescuer_limit_ms`: the time limit for records to be in the publishing\nstate. Defaults to 15 seconds. Notice that they may stay longer in this state\ndue to the interval.\n- `:garbage_collector_interval_ms`: the interval between garbage collection\nruns. Defaults to 30 seconds. Notice that it always runs on tree startup.\n- `garbage_colletor_limit_ms`: the time limit for published records to be in\nthe table. Defaults to 72 hours.\n- `query_opts`: Options to pass to Ecto queries. Defaults to [log: false]","ref":"Kafkaesque.html#start_link/1"},{"type":"module","title":"Kafkaesque.Acknowledger","doc":"Stage that updates in the database the messages that were published in Kafka\n\nTakes 3 options:\n- `:publisher_pid`: pid of the stage that will publish the messages.\n- `:repo`: the repo to execute the queries on.\n- `:query_opts`: A list of options sent to Repo calls.","ref":"Kafkaesque.Acknowledger.html"},{"type":"function","title":"Kafkaesque.Acknowledger.shutdown/2","doc":"","ref":"Kafkaesque.Acknowledger.html#shutdown/2"},{"type":"function","title":"Kafkaesque.Acknowledger.start_link/1","doc":"","ref":"Kafkaesque.Acknowledger.html#start_link/1"},{"type":"behaviour","title":"Kafkaesque.Client","doc":"Behaviour for clients","ref":"Kafkaesque.Client.html"},{"type":"callback","title":"Kafkaesque.Client.publish/2","doc":"","ref":"Kafkaesque.Client.html#c:publish/2"},{"type":"callback","title":"Kafkaesque.Client.start_link/1","doc":"","ref":"Kafkaesque.Client.html#c:start_link/1"},{"type":"type","title":"Kafkaesque.Client.client/0","doc":"","ref":"Kafkaesque.Client.html#t:client/0"},{"type":"module","title":"Kafkaesque.Clients.BrodClient","doc":"Basic client implementation using brod\n\nProduces synchronously, and automatically starts all the required producers.\n\nRequired options:\n- `:client_id`: An atom which will be used as the client name\n- `:brokers`: A list of {address, port} tuples for kafka brokers. Addresses\nshould be charlists.\n\nAlso accepts the same options as `:brod.start_client/3`","ref":"Kafkaesque.Clients.BrodClient.html"},{"type":"module","title":"Kafkaesque.GarbageCollector","doc":"Garbage collects messages that were already published\n\nTakes 3 options on startup:\n- `:repo`: the repo to perform garbage collection on\n- `:query_opts`: A list of options sent to Repo calls.\n- `:garbage_collector_interval_ms`: the interval between garbage collection\nruns. Notice that it always runs on startup.\n- `garbage_colletor_limit_ms`: the time limit for published records to be in\nthe table.","ref":"Kafkaesque.GarbageCollector.html"},{"type":"function","title":"Kafkaesque.GarbageCollector.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Kafkaesque.GarbageCollector.html#child_spec/1"},{"type":"function","title":"Kafkaesque.GarbageCollector.start_link/1","doc":"","ref":"Kafkaesque.GarbageCollector.html#start_link/1"},{"type":"module","title":"Kafkaesque.Message","doc":"Application-side representation of messages, published or not","ref":"Kafkaesque.Message.html"},{"type":"function","title":"Kafkaesque.Message.new/4","doc":"","ref":"Kafkaesque.Message.html#new/4"},{"type":"type","title":"Kafkaesque.Message.state/0","doc":"","ref":"Kafkaesque.Message.html#t:state/0"},{"type":"type","title":"Kafkaesque.Message.t/0","doc":"","ref":"Kafkaesque.Message.html#t:t/0"},{"type":"module","title":"Kafkaesque.Migrations","doc":"","ref":"Kafkaesque.Migrations.html"},{"type":"function","title":"Kafkaesque.Migrations.down/0","doc":"","ref":"Kafkaesque.Migrations.html#down/0"},{"type":"function","title":"Kafkaesque.Migrations.down/2","doc":"","ref":"Kafkaesque.Migrations.html#down/2"},{"type":"function","title":"Kafkaesque.Migrations.up/0","doc":"","ref":"Kafkaesque.Migrations.html#up/0"},{"type":"function","title":"Kafkaesque.Migrations.up/2","doc":"","ref":"Kafkaesque.Migrations.html#up/2"},{"type":"module","title":"Kafkaesque.Pipeline","doc":"This process just initializes the stages, linked to it. If one stage die,\nall stages die.\n\nIt also takes care of termination of the stages.","ref":"Kafkaesque.Pipeline.html"},{"type":"function","title":"Kafkaesque.Pipeline.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Kafkaesque.Pipeline.html#child_spec/1"},{"type":"function","title":"Kafkaesque.Pipeline.start_link/1","doc":"","ref":"Kafkaesque.Pipeline.html#start_link/1"},{"type":"module","title":"Kafkaesque.Producer","doc":"Queries the database for messages to be published\n\nTakes 2 options on startup:\n- `:producer_max_backoff_ms`: maximum time in milliseconds that the producer will\ntake between database reads. Notice that this is edge-case scenario and should\nonly happen when a) there are database issues or b) there are no new messages\nin the table for long.\n- `:repo`: the repo the producer will query on.\n- `:query_opts`: A list of options sent to Repo calls.","ref":"Kafkaesque.Producer.html"},{"type":"function","title":"Kafkaesque.Producer.shutdown/2","doc":"","ref":"Kafkaesque.Producer.html#shutdown/2"},{"type":"function","title":"Kafkaesque.Producer.start_link/1","doc":"","ref":"Kafkaesque.Producer.html#start_link/1"},{"type":"function","title":"Kafkaesque.Producer.stop_producing/1","doc":"","ref":"Kafkaesque.Producer.html#stop_producing/1"},{"type":"module","title":"Kafkaesque.Publisher","doc":"Stage that publishes messages in Kafka\n\nTakes 3 options:\n- `:repo`: the repo associated with the Kafkaesque instance\n- `:producer_pid`: pid of the stage that will produce the messages.\n- `:client`: the client module that will be used to publish the messages.\nDefaults to `Kafkaesque.Clients.BrodClient`.\n- `:client_opts`: A list of options to be passed to the client on startup.\nDefaults to `[]`. The default client requires options, so this can be\nconsidered required for most use-cases.\n- `:publisher_min_demand`: The minimum demand. See GenStage documentation for\nmore info. Defaults to 190.\n- `:publisher_min_demand`: The maximum demand. See GenStage documentation for\nmore info. Defaults to 200.","ref":"Kafkaesque.Publisher.html"},{"type":"function","title":"Kafkaesque.Publisher.shutdown/2","doc":"","ref":"Kafkaesque.Publisher.html#shutdown/2"},{"type":"function","title":"Kafkaesque.Publisher.start_link/1","doc":"","ref":"Kafkaesque.Publisher.html#start_link/1"},{"type":"module","title":"Kafkaesque.Rescuer","doc":"Rescues messages stuck in publishing state\n\nIf the shutdown wasn't graceful or there are errors, there is the possibility\nthat messages get lost in the `:publishing` state, which would stop the\npublishing for a topic + partition combination. This process rescues the\nstuck messages periodically.\n\n\nTakes 3 options on startup:\n- `:repo`: the repo to perform garbage collection on\n- `:query_opts`: A list of options sent to Repo calls.\n- `:rescuer_interval_ms`: the interval between garbage collection\nruns. Notice that it always runs on tstartup.\n- `rescuer_limit_ms`: the time limit for records to be in the publishing\nstate. Notice that they may stay longer in this state due to the interval.","ref":"Kafkaesque.Rescuer.html"},{"type":"function","title":"Kafkaesque.Rescuer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Kafkaesque.Rescuer.html#child_spec/1"},{"type":"function","title":"Kafkaesque.Rescuer.start_link/1","doc":"","ref":"Kafkaesque.Rescuer.html#start_link/1"},{"type":"module","title":"Kafkaesque.Supervisor","doc":"Supervisor for Kafkaesque","ref":"Kafkaesque.Supervisor.html"},{"type":"function","title":"Kafkaesque.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Kafkaesque.Supervisor.html#child_spec/1"},{"type":"function","title":"Kafkaesque.Supervisor.init/1","doc":"","ref":"Kafkaesque.Supervisor.html#init/1"},{"type":"function","title":"Kafkaesque.Supervisor.start_link/1","doc":"","ref":"Kafkaesque.Supervisor.html#start_link/1"},{"type":"extras","title":"Getting started","doc":"# Getting started\n\nFirst, you need to migrate your database to create the outbox. Create a\nmigration with Ecto and add the following:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddKafkaesque\n  def up do\n    Kafkaesque.Migrations.up()\n  end\n\n  def down do\n    Kafkaesque.Migrations.down()\n  end\nend\n```\n\nThen, you must define an outbox module, associated with the Repo you\nran the Kafkaesque migrations on. For example:\n\n```elixir\ndefmodule MyApp.Outbox do\n  use Kafkaesque, repo: MyApp.Repo\n\n  # Optional, sets the partition for the message. Should return an integer\n  # Defaults to returning 0.\n  def partition(_topic, _key, _body) do\n    Enum.random(0..1)\n  end\n\n  # Optional, encodes the body of the message, should return a String.\n  # Defaults to the identity function.\n  def encode(body) do\n    Jason.encode!(body)\n  end\nend\n```\n\nFinally, you should start a `Kafkaesque` instance in your supervision tree.\nFor example:\n\n```elixir\ndefmodule MyApp.Application do\n  def start(_type, _args) do\n    children = [\n      MyApp.Repo,\n      {MyApp.Outbox, [client_opts: [client_id: :my_client, brokers: [{\"localhost, 9092\"}]]]},\n    ]\n  end\nend\n```\n\nDone! Now you can publish messages. Try it out with:\n\n```elixir\nMyApp.Outbox.publish(\"topic\", \"mykey\", %{hello: :kafka})`\n```","ref":"getting-started.html"}],"content_type":"text/markdown"}